
/* Usage:

   Type mpz_redc_t contains a REDC ring.
   Initialized using mpz_redc_init_set.
   Cleared with mpz_redc_clear.

   mpz_redc_init_set (mpz_redc_t mrt, mpz_t m)
     Initialize mrt from m.  Now mrt can be used for computing mod m.

   mpz_redc_clear (mpz_redc_t mrt)
     Clear out mrt.

   mpz_redcify (mpz_t r, mpz_t u, mpz_redc_t mrt)
     Transform u into an REDC form and put it in r.  (Compute r = 2^n mod mrt,
     where n is log2(mrt).)
     After this call, r is suitable for calls to the fast mpz_modredc
     instead of mpz_mod.

   mpz_modredc (mpz_t r, mpz_t u, mpz_redc_t m)
     r <= u mod m.  requires that u has been converted by means of mpz_redcify.

   mpz_unredcify (mpz_t r, mpz_redc_t u)
     Transform u from REDC form and put it in r.
*/

#ifndef REDCIFY_H
#define REDCIFY_H

#include "gmp.h"
#include "gmp-impl.h"

#define MPZ(x) ((x)->_gmp_z)
#define BINV(x) ((x)->_gmp_binv)

#ifdef DOG
#define binvert_limb modlimb_invert /* backward compatibility */
#endif

typedef struct
{
  mp_limb_t _gmp_binv;
  mpz_t _gmp_z;
} mpz_redc_t[1];


static void
redcify (mp_ptr rp, mp_srcptr up, mp_size_t un, mp_srcptr mp, mp_size_t n);


void
mpz_redc_init_set (mpz_redc_t r, mpz_srcptr m);


void
mpz_redc_clear (mpz_redc_t mrt);


void
mpz_redcify (mpz_ptr r, mpz_srcptr u, mpz_redc_t mrt);


void
mpz_unredcify (mpz_ptr r, mpz_srcptr u, mpz_redc_t mrt);

void
mpz_modredc (mpz_ptr r, mpz_srcptr u, mpz_redc_t mrt);


#endif /* REDCIFY_H */
